(function(angular){

    angular.module('course-permissions.controllers', []).
        controller('PermissionsController', [
            '$scope',
            '$window',
            '$uibModal',
            '$http',
            '$q',
            '$sce',
            'CourseGroup',
            'CourseProfessor',
            'BasicCourseProfessor',
            'Groups',
            function(
                $scope,
                $window,
                $uibModal,
                $http,
                $q,
                $sce,
                CourseGroup,
                CourseProfessor,
                BasicCourseProfessor,
                Groups
            ) {

            var success_save_msg = 'Alterações salvas com sucesso.';
            var error_save_msg = 'Não foi possível salvar todas as alterações.';
            var cancel_changes_msg = 'Alterações canceladas.';

            $scope.permissions_changed = false;
            $scope.course_professors_to_delete = []
            $scope.course_professors_to_create = []
            $scope.course_professors_to_update = []

            $window.onbeforeunload = function (e) {
                if ($scope.permissions_changed) {
                    if (!confirm('Você possui alterações não salvas, tem certeza que deseja sair desta página e perder essas alterações?')) {
                        event.preventDefault();
                    }
                }
            };

            $scope.courseId = /course\/([^\/]+)\/permissions/.extract(location.pathname, 1);
            $scope.professors = BasicCourseProfessor.query({course: $scope.courseId, has_user: true}, function(professors){
                $scope.professors_before_changes = angular.copy(professors);
            });

            CourseGroup.get({id: $scope.courseId}, function(course){
                $scope.course_groups = course;
            });
            $scope.groups = Groups.query();

            $scope.get_as_safe_html = function(value) {
                return $sce.trustAsHtml(value);
            };

            $scope.contract_name = function(item) {
                if (item.contract && item.contract.name !== '')
                    return item.contract.name;
                else
                    return 'Grupos sem contrato';
            };

            $scope.group_label = function(item) {
                if (item.contract && item.contract.name != '')
                    return item.name + ' - ' + item.contract.name;
                else
                    return item.name;
            };

            $scope.save_permissions = function() {
                var promises_list = [];
                $scope.course_professors_to_delete.forEach(function(course_professor) {
                    course_professor = CourseProfessor.remove({id: course_professor.id});
                });
                $scope.course_professors_to_create.forEach(function(course_professor) {
                    course_professor_payload = angular.copy(course_professor);
                    course_professor_payload.user = course_professor.user.id;
                    course_professor = course_professor_payload.$save({
                        course: course_professor.course,
                        user: course_professor.user.id
                    });
                    promises_list.push(course_professor.$promise);
                });
                $scope.course_professors_to_update.forEach(function(course_professor) {
                    course_professor_payload = angular.copy(course_professor);
                    course_professor_payload.user = course_professor.user.id;
                    course_professor_payload.$update();
                    promises_list.push(course_professor_payload.$promise);
                });

                $q.all([promises_list]).then(function(results) {
                        $scope.alert.success(success_save_msg);
                        $scope.permissions_changed = false;
                        $scope.course_professors_to_delete = []
                        $scope.course_professors_to_create = []
                        $scope.course_professors_to_update = []
                    }, function(errorMsg) {
                        $scope.alert.error(error_save_msg);
                    }
                );

                $scope.course_groups.$update({id: $scope.courseId});
            };

            $scope.cancel_permissions_changes = function() {
                $scope.professors = $scope.professors_before_changes;
                $scope.permissions_changed = false;
                $scope.course_professors_to_delete = []
                $scope.course_professors_to_create = []
                $scope.course_professors_to_update = []
                $scope.alert.success(cancel_changes_msg);
            };

            $scope.remove_professor = function(course_professor, index) {
                var confirm_msg = 'Tem certeza que deseja remover este professor deste curso?' + 
                    ' Esta operação só será realizada após você salvar as alterações.'
                if(!confirm(confirm_msg)) return;
                $scope.professors.splice(index, 1);
                $scope.course_professors_to_delete.unshift(course_professor);
                $scope.permissions_changed = true;
            };

            $scope.update_professor_role = function(course_professor) {
                $scope.permissions_changed = true;
                $scope.course_professors_to_update.unshift(course_professor);
            };

            $scope.new_professors = function () {
                var modalInstance = $uibModal.open({
                    templateUrl: 'newProfessorModal.html',
                    controller: ['$scope', '$uibModalInstance', 'course_id', addProfessorsModalInstanceCtrl],
                    resolve: {
                        course_id: function () {
                            return $scope.course_id;
                        }
                    }
                });
                modalInstance.result.then(function (new_professors) {
                    angular.forEach(new_professors, function(professor){
                        var new_professor = new CourseProfessor();
                        new_professor.course =  $scope.courseId;
                        new_professor.user = professor.id;
                        new_professor.user = professor;
                        $scope.professors.unshift(new_professor);
                        $scope.course_professors_to_create.unshift(new_professor);
                        $scope.permissions_changed = true;
                    });
                });
            };
            var addProfessorsModalInstanceCtrl = function ($scope, $uibModalInstance, course_id) {

                $scope.new_professors = [];
                $scope.add_professors = function () {
                    $uibModalInstance.close($scope.new_professors);
                };

                $scope.cancel = function () {
                    $uibModalInstance.dismiss();
                };

                $scope.on_select_professor = function(model) {
                    $scope.new_professors.unshift(model);
                    $scope.asyncSelected = '';
                };

                $scope.remove_new_professor = function(index) {
                    $scope.new_professors.splice(index, 1);
                };

                $scope.getUsers = function(val) {
                    return $http.get(BASE_API_URL + '/user_search', {
                        params: {
                          name: val,
                          sensor: false
                        }
                    }).then(function(res){
                        var professors_found = [];
                        angular.forEach(res.data, function(item){
                            var formated_name = '';
                            if (item.first_name)
                                formated_name += item.first_name;
                            if (item.last_name)
                                formated_name = formated_name + ' ' + item.last_name;
                            if (formated_name)
                                formated_name = formated_name + ' - ';
                            formated_name += item.username;
                            if (item.email)
                                formated_name = formated_name + ' - ' + item.email;
                            item.formated_name = formated_name;
                            professors_found.push(item);
                        });
                        return professors_found;
                    });
                };
            };
        }
    ]);

})(angular);
