(function() {
    'use strict';

    angular
        .module('userParticipationGraphicsReport')
        .component('userParticipationGraphicsReport', {
            templateUrl: '/user-participation-graphics-report.template.html',
            controller: UserParticipationGraphicsReportCtrl,
            bindings: {
                date: '<',
            },
        });

    UserParticipationGraphicsReportCtrl.$inject = ['$scope', 'NteParticipationReport'];

    function UserParticipationGraphicsReportCtrl ($scope, NteParticipationReport) {

        var ctrl = this;

        ctrl.minimized = false;
        ctrl.ntes = [];

        ctrl.chart1Data = [];
        ctrl.chart1DatasetOverride = [
            {
                backgroundColor: '#89C9D5',
                barThickness: 10
            },
            {
                backgroundColor: '#1B7EC6',
                barThickness: 10
            },
            {
                backgroundColor: '#CCCCCC',
                barThickness: 10
            }
        ];
        ctrl.chart1Labels = [];
        ctrl.chart1Series = ['Ativos', 'Não ativos', 'Não participantes'];
        ctrl.chart1Options = {
            legend: {
                align: 'center',
                display: true,
                position: 'bottom'
            },
            plugins: {
                dataLabels: {
                    display: false
                }
            },
            scales: {
                xAxes: [{
                    stacked: true,
                }],
                yAxes: [{
                    stacked: true,
                    ticks: {
                        fontColor: 'black',
                        fontStyle: 'bold'
                    },
                }]
            },
        };

        ctrl.chart2Data = [];
        ctrl.chart2Labels = ['Ativos', 'Não ativos', 'Não participantes'];
        ctrl.chart2Options = {
            legend: {
                align: 'center',
                display: true,
                position: 'bottom'
            },
            plugins: {
                datalabels: {
                    color: '#fff',
                    display: true,
                    font: {
                        weight: 'bold',
                    },
                    formatter: (value, ctx) => {
                        const dataArr = ctx.chart.data.datasets[0].data;
                        const sum = dataArr.reduce((acc, curr) => acc + curr, 0);
                        const percentage = Math.round(value * 100 / sum) + "%";
                        return percentage;
                    }
                }
            }
        };

        ctrl.toggleDetails = toggleDetails;

        ctrl.$onChanges = function() {
            fetchNtes();
        };

        function fetchNtes() {
            NteParticipationReport.query({ date: ctrl.date }, (data) => {
                ctrl.ntes = data;

                ctrl.chart1Data = [
                    ctrl.ntes.map(function(nte) {
                        return nte.participation.actives;
                    }),
                    ctrl.ntes.map(function(nte) {
                        return nte.participation.not_actives;
                    }),
                    ctrl.ntes.map(function(nte) {
                        return nte.participation.not_involved;
                    }),
                ];
                ctrl.chart1Labels = ctrl.ntes.map(function(nte) {
                    return nte.name;
                });

                ctrl.chart2Data = ctrl.ntes.reduce(function (acc, nte) {
                    return [
                        acc[0] + nte.participation.actives,
                        acc[1] + nte.participation.not_actives,
                        acc[2] + nte.participation.not_involved
                    ];
                }, [0, 0, 0]);
            });
        }

        function toggleDetails() {
            ctrl.minimized = !ctrl.minimized;
        }
    }
})();
